/* bmp.c
 * Fonctions permettant la lecture d'images BMP
 *
 * Auteur: VOURIOT Thierry
 * Web: http://thierry.vouriot.free.fr
 * Email: thierry.vouriot@free.fr
 * Date: 21 aot 2002
 */
 
/* Structure d'un fichier BMP 24 bits
 * Le format BMP est divis en 3 parties le header,
 * la zone bitmap info et les donnes
 *
 * Le header
 * Position		Type		Description
 * 00h			DB "BM"		BM indique qu'il s'agit d'un fichier bitmap
 * 02h			DD ?		Taille du fichier en octets
 * 06h			DW 0		Rserv
 * 08h			DW 0		Rserv
 * 0Ah			DD ?		Offset des donnes de l'image
 *
 * La zone bitmap info
 * Position		Type		Description
 * OEh			DD ?		Taille de la zone bitmap info
 * 12h			DD ?		Largeur de l'image en pixels
 * 16h			DD ?		Hauteur de l'image en pixels
 * 1Ah			DW 1		Nombre de plans (1)
 * 1Ch			DW 1|4|8|24	Bits par pixel
 * 1Eh			DD 0|1|2	Format de compression (O=non compresse)
 * 22h			DD ?		Taille de l'image en octets
 * 26h			DD ?		Rsolution horizontale en pixels par mtre
 * 2Ah			DD ?		Rsolution verticale en pixels par mtre
 * 2Eh			DD ?		Nombre de couleurs utilises  ...
 * 32h			DD ?		Nombre d'index de couleurs importants pour l'affichage
 *
 */

#include "bmp.h"

#define CTOI(C) (*(int*)&C)	//rcupre en int un nombre point par un char*

/* Fonction lisant un fichier BMP 24 ou 8 bits et remplissant la structure
 * BMPFILE passe en paramtre
 */
int LoadBMPFile(char *filename, BMPFILE *bmpFile)
{
	FILE *filePtr;			// pointeur sur le fichier
	unsigned char header[0x36];	// tableau qui contiendra l'en-tte du fichier BMP
	int colorMode;			// 3 pour RGB
	int imagePos;			// offset du dbut des donnes de l'image
	long imageSize;			// taille de l'image BMP
	long imageIdx;			// compteur
	unsigned char colorSwap;	// variable d'change
	
	// ouverture du fichier BMP
	if (!(filePtr = fopen(filename, "rb"))) {
		fclose(filePtr);
        	return BMP_FILE_NOT_FOUND;
	}	
	// lecture de l'en-tte
	if (fread(header,1,0x36,filePtr)!=0x36) {
		fclose(filePtr);
		return BMP_BAD_IMAGE_TYPE;
	}	
	// BM indique que l'on se trouve en prsence d'un fichier bitmap
	if (header[0]!='B' || header[1]!='M') {
		fclose(filePtr);
		return BMP_BAD_IMAGE_TYPE;
	}
	// indique que l'image n'est pas compresse	
	if (CTOI(header[0x1E])!=0) {
		fclose(filePtr);
		return BMP_BAD_IMAGE_TYPE;
	}
	// image 24 et 8 bits uniquement
	bmpFile->bitCount = CTOI(header[0x1C]);
	if (bmpFile->bitCount!=24 && bmpFile->bitCount!=8) {
		fclose(filePtr);
		return BMP_BAD_BITS;
	}
	colorMode = bmpFile->bitCount / 8;

	// rcupre les infos sur les donnes de l'image
	imagePos = CTOI(header[0x0A]);
	imageSize = CTOI(header[0x22]);
	// rcupre les dimensions de l'image
	bmpFile->imageWidth = CTOI(header[0x12]);
	bmpFile->imageHeight = CTOI(header[0x16]);	
		
	// si les infos sur les donnes de l'image n'ont pas t rcupr
	// on les calcule
	if (imageSize==0)
		imageSize=bmpFile->imageHeight*bmpFile->imageWidth*colorMode;
	if (imagePos==0)
		imagePos=0x36;

	// charge les donnes de l'image
	fseek(filePtr,imagePos,0);
	bmpFile->imageData = (unsigned char*) malloc(sizeof(unsigned char) * imageSize);
	
	if (!bmpFile->imageData) {
		fclose(filePtr);
		return BMP_BAD_DATA;
	}

	if (fread(bmpFile->imageData,1,imageSize,filePtr)!=imageSize) 
	{
		free(bmpFile->imageData);
		fclose(filePtr);
		return BMP_BAD_DATA;
	}
	
	fclose(filePtr);

	// dans les images BMP les couleurs des pixels sont en mode BGR
	// transformation en mode RGB
	if (bmpFile->bitCount == 24)
	for (imageIdx=0; imageIdx < imageSize; imageIdx += colorMode)
	{
		colorSwap = bmpFile->imageData[imageIdx];
		bmpFile->imageData[imageIdx] = bmpFile->imageData[imageIdx + 2];
		bmpFile->imageData[imageIdx + 2] = colorSwap;
	}
	
	// Format de l'image RGB ou niveau de gris
	switch (bmpFile->bitCount) {
		case 8: bmpFile->texFormat = GL_LUMINANCE; break;
		case 24: bmpFile->texFormat = GL_RGB; break;
	}
	
	return 1;
}

